@@ -17,8 +17,6 @@
 #ifndef __UCONFIG_H__
 #define __UCONFIG_H__
 
-/* -DU_DISABLE_RENAMING=1 */
-#define U_DISABLE_RENAMING 1
 
 /*!
  * \file
@@ -91,7 +89,7 @@
  * @internal
  */
 #ifndef U_DISABLE_RENAMING
-#define U_DISABLE_RENAMING 0
+#define U_DISABLE_RENAMING 1
 #endif
 
 /**
@@ -455,4 +453,4 @@
 #   define UCONFIG_NO_FILTERED_BREAK_ITERATION 0
 #endif
 
-#endif
+#endif  // __UCONFIG_H__
